close all; % CLOSE FIRST
clear all; % THEN CLEAR VARIABLES
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Real-Time Julia Set And Mandelbrot Set Demo
% Version 1.2
% Changes: 
% Added GPU Based Image Filtering on the Iteration Data
% Added Colormap choosing
% Added Panning at a great pain in coding.
% Bug Fixes and Updates
%
% Computation accelerated by GPU (CUDA C/C++)
% CPU Support has been added (Still slow) and is uded by default.
% To switch to GPU, set S.UseCPU = 0; Hope that clarifies this.
% Tested In Windows 7 64bit R2017a,intel i5-750 & nVidia GTX960
%
% Requirements: CUDA Capable Graphics with Compute Capability 3.0+ and
% Reasonable amount of GPU Memory based on display resolution (4K Fullscreen Might Require a 4GB GPU)
% Demo should work on Geforce 6XX and higher.
% Probably Matlab R2017a version 
% In case of a failure: Try Installing MS Redistributables for Visual
% Studio 2015. In case it fails, Try also 2017 versions.
% In case there is still some error,try updating your nVidia Video Drivers.
% As a next step,feel free to contact me with the error description.
% Note: Demo might require R2017a and newer version. This has not been
% tested. It might work on older versions,but I personally wouldnt count on
% that. I can recompile the project for R2015a and R2016a versions.
%
% Demo is Freely available for non-commercial Usage
% Author: Vojtech Ters 2017.12.30
% https://www.beechwood.eu
% vojta.ters@beechwood.eu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add release Directory to Matlab Path
MexFilePath = ['..',filesep,'x64',filesep,'Release'];
addpath(MexFilePath);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constants and Settings (Take Care When Editing!) %
global S;
S.UseCPU = 0;               % Either use CPU [1] or GPU [0] For Computing 
S.RelativeChange = 0.02;    % Required Relative Change of the parameters on slider Callback
S.Spacing = 0.04;           % General Normalized spacing [-]
S.EditWidth  = 0.8;         % Normalized EditBox Width [-]
S.EditHeight = 0.07 ;       % Normalized EditBox Height [-]
S.ButtonGroupWidth = 0.25;  % Total normalized width of Labels and Boxes [-]
S.BGSpacing = 0.01;         % Spacing For the ButtonGroup [-]
S.FontSize = 13;            % FontSize to use [-]
S.AxSpacing = 0.15;         % Normalized distance of Chart-PanelBorder [-] 
S.SlideEditRatio = 4;       % Amount of "Edit Components's heigh" of the Slider
S.ZoomFactor = 1.2;         % Zoom out Factor (Free to change to your needs) 
S.PanFactor = 1e-2;         % Pan Factor (How Much Does the axes move)
S.EnableGaussBlur = 1;      % Enables some small amout of gaussian blur (Matlab-Based Processing)
S.Kernel = fspecial('gaussian',[20,18],0.7); % Custom Image Filter 
S.Kernel = single(S.Kernel);                 % Must Be in Single

%%%%%%%%%%%%%%%%%%%%%%%%
% Default Fulia Values %
S.Threshold = 2;
S.NumIter = 100;
S.JuliaVal = -0.8 + 0.156i;
S.SetType = 'Julia';
%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get Display Resolution %
S.Displ = groot();
S.ScW = S.Displ.ScreenSize(3);
S.ScH = S.Displ.ScreenSize(4);
S.SlideHeigh = S.SlideEditRatio * S.EditHeight;
S.FiW = 1200;     % Current Figure Width
S.FiH = 720;      % Current Figure Height

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definition of Plane for Visualization          %
S.XMin = -2.0;      % Complex - X - Axis Minimum %
S.XMax = +2.0;      % Complex - X - Axis Maximum %
S.YMin = -2.0;      % Complex - Y - Axis Minimum %
S.YMax = +2.0;      % Complex - Y - Axis Maximum %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create Information Figure %
S.FiWi = 720;
S.FiHi = 460; 
S.Fi = figure('name','Program info');
S.Fi.Position = [S.ScW/2-S.FiW/2,S.ScH/2-S.FiH/2,S.FiWi,S.FiHi];
S.Fi.Visible = 'off';
S.Fi.NumberTitle = 'off';
S.Fi.MenuBar = 'none';
S.Fi.ToolBar = 'none';
S.Fi.CloseRequestFcn = {@Callback_InfoTool};
 
 % ADD INFO TO THE FIGURE
 FileID = fopen('Info.txt');
 Inn = []; textline = fgets(FileID);
 while ischar(textline)
     Inn = [Inn,textline];
     textline = fgets(FileID);
 end
 fclose(FileID);
 
 S.SimText = uicontrol(S.Fi,'Style','text',...
     'Units','normalized',...
     'Position',[S.Spacing,S.Spacing,1-2*S.Spacing,1-2*S.Spacing],...
     'Tag','KeepItSimple',...
     'String',Inn,...
     'HandleVisibility','on',...
     'HorizontalAlignment','Left');
 % CHANGE ICON
 warning('off','MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame');
 javaFrame = get(S.Fi,'JavaFrame');
 javaFrame.setFigureIcon(javax.swing.ImageIcon('Moon.png'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Create Main Figure %%%%%%% 
S.Fm = figure('name','Realtime Julia');
S.Fm.Position = [S.ScW/2-S.FiW/2,S.ScH/2-S.FiH/2,S.FiW,S.FiH];
S.Fm.NumberTitle = 'off';
S.Fm.Color = 'White';
S.Fm.Tag = 'MainFigure';
S.Fm.MenuBar = 'none';
S.Fm.ToolBar = 'figure';
S.Fm.CloseRequestFcn = {@Callback_MainFigClose};
S.Fm.ResizeFcn = {@Callback_Resize};
S.Fm.SizeChangedFcn = {@Callback_Resize};
warning('off','MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame');
javaFrame = get(S.Fm,'JavaFrame');
javaFrame.setFigureIcon(javax.swing.ImageIcon('Moon.png'));
set(S.Fm,'doublebuffer','off');

% %%%%% GET RID OFF TOLLBAR COMPONENTS AND SEPARATORS FROM MAIN FIGURE
delete(findall(S.Fm,'ToolTipString','Open File'));
delete(findall(S.Fm,'ToolTipString','Print Figure'));
delete(findall(S.Fm,'ToolTipString','New Figure'));
delete(findall(S.Fm,'ToolTipString','Hide Plot Tools'));
delete(findall(S.Fm,'ToolTipString','Show Plot Tools and Dock Figure'));
delete(findall(S.Fm,'ToolTipString','Link Plot'));
delete(findall(S.Fm,'ToolTipString','Brush/Select Data'));
delete(findall(S.Fm,'ToolTipString','Edit Plot'));
delete(findall(S.Fm,'ToolTipString','Insert Legend'));
delete(findall(S.Fm,'ToolTipString','Insert ColorBar'));
%delete(findall(S.Fm,'ToolTipString','Pan')); % This is annoying
Tb = findall(S.Fm,'tag','FigureToolBar');
Ty = findall(Tb,'Type','uitoggletool');
set(Ty,'Separator','off');
S.Panning = 0;


%%%%%%%%%%%%%%%%%%%%
% Add Info Toolbar %
[img,map] = imread('EarthX.gif');
ICData = ind2rgb(img,map);
itool = uipushtool('Parent',Tb,...
    'TooltipString','About',...
    'ClickedCallBack',{@Callback_InfoTool},...
    'Tag','Info Box',...
    'HandleVisibility','on',...
    'CData',ICData);

S.ZoomHandle = zoom;                                % Axes Zooming Handle
S.PanHandle = pan;                                  % Axes Pan Handle
%ZoomIn = findall(S.Fm,'ToolTipString','Zoom In');   % ZoomIn ToolBarHandle
%ZoomOut = findall(S.Fm,'ToolTipString','Zoom Out'); % ZoomOut TolBar Handle

%%%%%%%%%%%%%%%%%%%%%%
% Load Initial Image %
IM = im2double(imread('Waterfall.jpg'));
IM = imresize(IM,[S.FiH,S.FiW]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add Visualization Chart %
GrayC = 0.7;
S.VisIm = axes();                 % Axes handle
S.VisIm.Color = GrayC*ones(1,3);  % Axes Background color 
S.VisIm.Clipping = 'off';         % Annnoying
S.VisIm.BoxStyle = 'full';        
S.VisIm.XLim = [S.XMin, S.XMax];
S.VisIm.YLim = [S.YMin, S.YMax];
S.VisIm.Position = [S.ButtonGroupWidth + 2* S.BGSpacing,S.BGSpacing,1 - 3*S.BGSpacing - S.ButtonGroupWidth,1-2*S.BGSpacing];    
S.VisImSC = imagesc(S.VisIm,[S.XMin,S.XMax],[S.YMin,S.YMax],IM);
S.VisIm.XLabel = [];           % Disable X Labels
S.VisIm.YLabel = [];           % Disable Y Labels   
S.VisIm.XTick = [];            % Disable X-Ticks
S.VisIm.YTick = [];            % Disable Y-Ticks
S.PrevXLim = S.VisIm.XLim;
S.PrevYLim = S.VisIm.YLim;



%% %%%%%%%% ADD CONTROLS AND METHODS TO MAIN WINDOW
VertC = 1;
S.NumControls = 6 + 4; 
S.ContWidth = (1 - 3*S.Spacing)/2; 
S.ContHeigh = (1 - (S.NumControls+1)*S.Spacing)/S.NumControls;
S.ButtonG = uibuttongroup('Title','General Config','TitlePosition','CenterTop'); 
S.ButtonG.Position = [S.BGSpacing,S.BGSpacing,S.ButtonGroupWidth,1 - 2*S.BGSpacing];

%%%%%%% JULIA COMPLEX VALUE %%%%%%%
S.EditJuliaR = uicontrol('style','Edit','parent',S.ButtonG,'string',num2str(real(S.JuliaVal)),'units','normalized');
S.EditJuliaI = uicontrol('style','Edit','parent',S.ButtonG,'string',num2str(imag(S.JuliaVal)),'units','normalized');
S.LabJulia = uicontrol('style','Text','parent',S.ButtonG,'string','Julia Set [R | I]','units','normalized');
S.EditJuliaR.Position = [2*S.Spacing + S.ContWidth                              , 1 - S.EditHeight*VertC,S.ContWidth / 2 - S.Spacing / 2,S.ContHeigh];
S.EditJuliaI.Position = [3*S.Spacing + S.ContWidth + S.ContWidth/2 - S.Spacing/2, 1 - S.EditHeight*VertC,S.ContWidth / 2 - S.Spacing / 2,S.ContHeigh];
S.LabJulia.Position   = [S.Spacing                                              , 1 - S.EditHeight*VertC,S.ContWidth                    ,S.ContHeigh];
VertC = VertC + 1;

%%%%%%% COMPLEX SET CHOOSING %%%%%%%
S.ListMethod    = uicontrol('style','listbox','parent',S.ButtonG,'units','normalized');
S.LabListMethod = uicontrol('style','Text','parent',S.ButtonG,'string','Complex Set','units','normalized');
S.ListMethod.Position = [2*S.Spacing+S.ContWidth,1-S.EditHeight*VertC,S.ContWidth,S.ContHeigh];
S.LabListMethod.Position = [S.Spacing,1-S.EditHeight*VertC,S.ContWidth,S.ContHeigh];
S.ListMethod.String = {'Julia Set';'Mandelbrot Set'};
VertC = VertC + 2;

%%%%%%% COLORMAP CHOOSING %%%%%%%
S.ListColormap    = uicontrol('style','listbox','parent',S.ButtonG,'units','normalized');
S.LabListColormap = uicontrol('style','Text','parent',S.ButtonG,'string','Image Colormap','units','normalized');
S.ListColormap.Position = [2*S.Spacing+S.ContWidth,1-S.EditHeight*VertC,S.ContWidth,2*S.ContHeigh];
S.LabListColormap.Position = [S.Spacing,1-S.EditHeight*VertC,S.ContWidth,2*S.ContHeigh];
S.ListColormap.String = {'Hot';'Jet';'HSV';'Bone';'MorgenStemning'};
VertC = VertC + 1;


%% %%%%%%% JULIA REAL & IMAGINARY SLIDER LABLES %%%%%%%
S.LabJuliaSlideR = uicontrol('style','Text','parent',S.ButtonG,'units','normalized','String','Julia Real');
S.LabJuliaSlideI = uicontrol('style','Text','parent',S.ButtonG,'units','normalized','String','Julia Imag');
S.LabJuliaSlideR.Position = [S.Spacing                , 1 - S.EditHeight*VertC  ,S.ContWidth,S.ContHeigh];
S.LabJuliaSlideI.Position = [2*S.Spacing + S.ContWidth, 1 - S.EditHeight*VertC ,S.ContWidth,S.ContHeigh];

% F Stands for "Fake" as the default matlab sliders are ugly
S.JuliaSlideRF          = uicontrol('style','slider','parent',S.ButtonG,'units','normalized');
S.JuliaSlideIF          = uicontrol('style','slider','parent',S.ButtonG,'units','normalized');
S.JuliaSlideRF.Position = [S.Spacing                , 1 - S.EditHeight*VertC - S.SlideHeigh ,S.ContWidth,S.SlideHeigh];
S.JuliaSlideIF.Position = [2*S.Spacing + S.ContWidth, 1 - S.EditHeight*VertC - S.SlideHeigh ,S.ContWidth,S.SlideHeigh];
S.JuliaSlideRF.Visible  = 'off';
S.JuliaSlideIF.Visible  = 'off';

% CREATE UNDOCUMENTED JSLIDER (Because its actually a slider and not a scrollbar)
S.JuliaSlideR = javax.swing.JSlider;
S.JuliaSlideI = javax.swing.JSlider;
set(S.JuliaSlideR,'minimum',-2000,'maximum',2000,'MinorTickSpacing',1, 'MajorTickSpacing',1000 , 'PaintLabels',true,'orientation',1,'Value',real(S.JuliaVal*1000));
set(S.JuliaSlideI,'minimum',-2000,'maximum',2000,'MinorTickSpacing',1, 'MajorTickSpacing',1000 , 'PaintLabels',true,'orientation',1,'Value',imag(S.JuliaVal*1000));
[~,S.HandleSlideJuliaR] = javacomponent(S.JuliaSlideR,getpixelposition(S.JuliaSlideRF,true));
[~,S.HandleSlideJuliaI] = javacomponent(S.JuliaSlideI,getpixelposition(S.JuliaSlideIF,true));
VertC = VertC + 1;S.SlideEditRatio;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set Custom Labels on the Ticks %
Labels = {'-2', '-1', '0', '1', '2'};
Values = {-2000,-1000,0,1000,2000}; 
LabelTable = get(S.JuliaSlideR, 'LabelTable');
for i = 1:length(Labels)
    key = java.lang.Integer(Values{i});
    val = LabelTable.get(key);
    val.setText(Labels{i});
    LabelTable.put(key, val);
end
set(S.JuliaSlideR, 'LabelTable', LabelTable);
set(S.JuliaSlideI, 'LabelTable', LabelTable);


%% %%%%%%% THRESHOLD AND ITERATION SLIDERS AND LABELS %%%%%%%

S.LabSlideThresh = uicontrol('style','Text','parent',S.ButtonG,'units','normalized','String','Threshold');
S.LabSlideIter   = uicontrol('style','Text','parent',S.ButtonG,'units','normalized','String','NumIter');
S.LabSlideThresh.Position = [S.Spacing                , (1 - S.EditHeight*VertC - S.SlideHeigh),S.ContWidth,S.ContHeigh];
S.LabSlideIter.Position   = [2*S.Spacing + S.ContWidth, (1 - S.EditHeight*VertC - S.SlideHeigh),S.ContWidth,S.ContHeigh];

% F Stands for "Fake" as the default matlab sliders are ugly
S.SlideThresF          = uicontrol('style','slider','parent',S.ButtonG,'units','normalized');
S.SlideIterF           = uicontrol('style','slider','parent',S.ButtonG,'units','normalized');
S.SlideThresF.Position = [S.Spacing                , 1 - S.EditHeight*VertC - 2 * S.SlideHeigh ,S.ContWidth,S.SlideHeigh];
S.SlideIterF.Position  = [2*S.Spacing + S.ContWidth, 1 - S.EditHeight*VertC - 2 * S.SlideHeigh ,S.ContWidth,S.SlideHeigh];
S.SlideThresF.Visible  = 'off';
S.SlideIterF.Visible   = 'off';

S.SlideThresh = javax.swing.JSlider;
S.SlideIter   = javax.swing.JSlider;
set(S.SlideThresh,'minimum',0,'maximum',5000 ,'MinorTickSpacing',1, 'MajorTickSpacing',1000 , 'PaintLabels',true,'orientation',1,'Value',S.Threshold*1000);
set(S.SlideIter ,'minimum',0,'maximum',1000,'MinorTickSpacing',1,'MajorTickSpacing',500, 'PaintLabels',true,'orientation',1,'Value',S.NumIter);
[~,S.HandleSlideThresh] = javacomponent(S.SlideThresh,getpixelposition(S.SlideThresF,true));
[~,S.HandleSlideIter] = javacomponent(S.SlideIter,getpixelposition(S.SlideIterF,true));
VertC = VertC + S.SlideEditRatio;

% SET CUSTOM LABELS ON THE TICKS
Labels = {'0', '1', '2', '3', '4','5'};
Values = {0,1000,2000,3000,4000,5000}; 
LabelTable = get(S.SlideThresh, 'LabelTable');
for i = 1:length(Labels)
    key = java.lang.Integer(Values{i});
    val = LabelTable.get(key);
    val.setText(Labels{i});
    LabelTable.put(key, val);
end
set(S.SlideThresh, 'LabelTable', LabelTable);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Assign Function CallBacks %%%%%%%
S.EditJuliaR.Callback           = @Callback_Generic;   % Custom Julia Value Change Callback
S.EditJuliaI.Callback           = @Callback_Generic;   % Custom Julia Value Change Callback
S.ListMethod.Callback           = @Callback_Generic;   % Method Change Callback
S.ListColormap.Callback         = @Callback_Colormap;   % Colormap Change CallBack
S.ZoomHandle.ActionPostCallback = @Callback_PostZoom;   % Post Zoom only
S.PanHandle.ActionPostCallback  = @Callback_PostPan;    % Panning Stops
S.PanHandle.ActionPreCallback   = @Callback_PrePan;     % Panning Starts
S.Fm.WindowButtonMotionFcn      = @Callback_Panning;    % Set the motion detector.

%%%%%%%%%%%%%%%%%%%%%%%
% Undocumented Matlab %
HJuliaR = handle(S.JuliaSlideR, 'CallbackProperties');
HJuliaI = handle(S.JuliaSlideI, 'CallbackProperties');
HIter   = handle(S.SlideIter,   'CallbackProperties');
HThresh = handle(S.SlideThresh, 'CallbackProperties');

HJuliaR.StateChangedCallback    = @Callback_Slider;
HJuliaI.StateChangedCallback    = @Callback_Slider;
HIter.StateChangedCallback      = @Callback_Slider;
HThresh.StateChangedCallback    = @Callback_Slider;









