%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @brief Called on After-Panning of the Axes
% https://www.iriscores.com
% Author: Vojtech Ters
% vojta.ters@iriscores.com
% Material free for non-commercial use.
% Please report any Bugs you find.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Callback_Panning( src,Event )
    
    global S;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Proceed Only When Panning %
    if(S.Panning == 0)
        return;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Check if Position has Changed %
    CurrentPos = get(0, 'PointerLocation');
    DiffX = CurrentPos(1) - S.CursorPos(1);
    DiffY = CurrentPos(2) - S.CursorPos(2);
    if((CurrentPos(1) == S.CursorPos(1)) && (CurrentPos(2) == S.CursorPos(2)))
        return;
    end
    
    S.CursorPos = CurrentPos;
    PanResX = S.XMax - S.XMin;
    PanResY = S.YMax - S.YMin;
    S.XMin = S.XMin - S.PanFactor * PanResX * DiffX;
    S.XMax = S.XMax - S.PanFactor * PanResX * DiffX;
    S.YMin = S.YMin + S.PanFactor * PanResY * DiffY;
    S.YMax = S.YMax + S.PanFactor * PanResY * DiffY;

    %%%%%%%%%%%%%%%%%%%%%%%%%%
    % Grab Figure Dimensions %
    Nx = S.Fm.Position(3);
    Ny = S.Fm.Position(4);
    

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Do not Try to compute poutside the default recommended ranges [-2,2] %
    if(S.XMin < -2)
        S.XMin = -2;
    end
    if(S.XMax > 2)
        S.XMax = 2;
    end
    if(S.YMin < -2)
        S.YMin = -2;
    end
    
    if(S.YMax > 2)
        S.YMax = 2;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Call Drawing Function to Redraw with new Settings %
    DrawSet(Nx,Ny,S.VisImSC,S.NumIter,S.Threshold,S.JuliaVal,S.SetType,S.EnableGaussBlur,S.UseCPU);

  
end
