%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @brief Called on After-Zooming
% https://www.iriscores.com
% Author: Vojtech Ters
% vojta.ters@iriscores.com
% Material free for non-commercial use.
% Please report any Bugs you find.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Callback_PostZoom( src,Event )
    
    global S;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Get Set type (Julia / Mandel) %
    S.SetType = S.ListMethod.String{S.ListMethod.Value};
    
    %%%%%%%%%%%%%%%%%%%%
    % Grab Julia Value %
    JuliaER = str2double(S.EditJuliaR.String);
    JuliaEI = str2double(S.EditJuliaI.String);
    S.JuliaVal = complex(JuliaER,JuliaEI);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Setup Slider Julia Value %
    set(S.JuliaSlideR,'Value',real(S.JuliaVal*1000));
    set(S.JuliaSlideI,'Value',imag(S.JuliaVal*1000));

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    % Grab Figure Dimensions %
    Nx = S.Fm.Position(3);
    Ny = S.Fm.Position(4);
    
    %%%%%%%%%%%%%%%%%%%%%%%
    % Fix Matlab Zoom-Out %
    DiffX = S.VisIm.XLim(2) - S.VisIm.XLim(1);
    DiffY = S.VisIm.YLim(2) - S.VisIm.YLim(1);
    if(S.PrevXLim(2) < S.VisIm.XLim(2))
        S.XMin = S.XMin - DiffX / S.ZoomFactor;
        S.XMax = S.XMax + DiffX / S.ZoomFactor;
        S.YMin = S.YMin - DiffY / S.ZoomFactor;
        S.YMax = S.YMax + DiffY / S.ZoomFactor;
    else
       % ZOOMIN IS WORKING ALRIGHT
       S.XMin = S.VisIm.XLim(1);
       S.XMax = S.VisIm.XLim(2);
       S.YMin = S.VisIm.YLim(1);
       S.YMax = S.VisIm.YLim(2);
    end
    
    S.PrevXLim = S.VisIm.XLim;
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Do not Try to compute poutside the default recommended ranges [-2,2] %
    if(S.XMin < -2)
        S.XMin = -2;
    end
    if(S.XMax > 2)
        S.XMax = 2;
    end
    if(S.YMin < -2)
        S.YMin = -2;
    end
    
    if(S.YMax > 2)
        S.YMax = 2;
    end
   

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Call Drawing Function to Redraw with new Settings %
    DrawSet(Nx,Ny,S.VisImSC,S.NumIter,S.Threshold,S.JuliaVal,S.SetType,S.EnableGaussBlur,S.UseCPU);
    
  
end
