%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @brief Callback for UI Sliders
% https://www.iriscores.com
% Author: Vojtech Ters
% vojta.ters@iriscores.com
% Material free for non-commercial use.
% Please report any Bugs you find.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Callback_Slider( src,Event )
   
    global S;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % GET SLIDER VALUES (Some Slider Values are x1000!) %
    JuliaRS      = get(S.JuliaSlideR,'Value')/1000;
    JuliaIS      = get(S.JuliaSlideI,'Value')/1000;
    Thresh       = get(S.SlideThresh,'Value')/1000;
    NumIter      = get(S.SlideIter,'Value');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    % At Least One Iteration %
    if(NumIter <= 1)
        NumIter = 1;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Minimum Slider Resolution greater than zero %
    if(Thresh <= 1e-3)
        Thresh = 1e-3;
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Stop Evaluating Callbacks too Frequently %
    Evaluate = false;
    RealRel   = abs((JuliaRS - real(S.JuliaVal)) / real(S.JuliaVal));
    ImagRel   = abs((JuliaIS - imag(S.JuliaVal)) / imag(S.JuliaVal));
    ThreshRel = abs((Thresh - S.Threshold) / S.Threshold);
    IterRel   = abs((NumIter - S.NumIter) / S.NumIter);
    
    if(RealRel > S.RelativeChange)
        Evaluate = true;
    end
    if(ImagRel > S.RelativeChange)
        Evaluate = true;
    end
    
    if(ThreshRel > S.RelativeChange)
        Evaluate = true;
    end
    
    if(IterRel > S.RelativeChange)
        Evaluate = true;
    end
    
    if(Evaluate)
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%
        % Update Global Settings %
        S.NumIter = NumIter;
        S.Threshold = Thresh;
        S.JuliaVal = complex(JuliaRS,JuliaIS);
        S.SetType = S.ListMethod.String{S.ListMethod.Value};

        set(S.EditJuliaR,'String',num2str(real(S.JuliaVal)));
        set(S.EditJuliaI,'String',num2str(imag(S.JuliaVal)));

        %%%%%%%%%%%%%%%%%%%%%%%%%%
        % Grab Figure Dimensions %
        Nx = S.Fm.Position(3);
        Ny = S.Fm.Position(4);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Call Drawing Function to Redraw with new Settings %
        DrawSet(Nx,Ny,S.VisImSC,S.NumIter,S.Threshold,S.JuliaVal,S.SetType,S.EnableGaussBlur,S.UseCPU);
        
    end
end
