%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @brief Computes the Mandelbrot Set given all required Parameters
% @param Nx : Amount of points in the X - Dimension
% @param Ny: Amount of points in the Y - Dimension
% @param AxesHandle: gca or custom axes handle.
% @param ImHandle: Image Axis Handle
% @param NumIter: Amount of Iterations to Process
% @param Threshold: Iteration threshold 
% @param JuliaVal: Julia Set value (Not used in case of Mandlbrot)
% @param SetType: Either Julia Set or MandelBrot Set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function DrawSet(Nx,Ny,ImHandle,NumIter,Threshold,JuliaVal,SetType,GaussBlur,UseCPU)


if(isMultipleCall())
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Multiple Calls of a slider callback %
    return;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
CudaChunk = 100; % Recommended amount of Iteration Calculations per Cuda kernel call 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global S;
xStart = S.XMin;
xStop = S.XMax;
yStart = S.YMin;
yStop = S.YMax;

if(xStop < xStart)
   %%%%%%%%%%%%%%%%%%%%%%%
   % Restart Application %
   error('X - Axis Failure!')
end
if(yStop < yStart)
    %%%%%%%%%%%%%%%%%%%%%%%
    % Restart Application %
    error('Y - Axis Failure!');
end


if(strcmpi(SetType,'Julia Set'))
    Set = 'Julia';
elseif(strcmpi(SetType,'Mandelbrot Set'))
    Set = 'Mandel';
else
    error('unknown fractal set!');
end


if(UseCPU)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% Compute the Set Via CPU %%%%%%%
    [ Iter,FunctionData ] = CPUFractals( Nx,Ny,NumIter,Threshold,JuliaVal,Set,xStart,xStop,yStart,yStop );
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% Custom Image Filter (OPTIONAL) %%%%%%%
    if(GaussBlur)
        Iter = imfilter(Iter,double(S.Kernel));
    end
    
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% Compute Set With CUDA Acceleration %%%%%%%
    [Iter,FunctionData] = QJulia(Set,'ABS',[Nx,Ny,xStart,xStop,yStart,yStop,Threshold,NumIter,CudaChunk,real(JuliaVal),imag(JuliaVal),GaussBlur],S.Kernel);
end


% Image Postprocess (Optionall,can be just Amount of Iterations)
%Image = FunctionData.*Iter;
Image = Iter;


% Note: Deleting and Creating new Axes is time consuming
% But: it works! I have tried plenty of options including
% setting each of the required variables:
% S.VisImSC.XData | YData
% S.VisImSC.CData
% S.VisIm.XLim | YLim
% But none of them worked as expected. I guess,there is some mechanism that
% Changes the parent / child object properties of the Axes / Image. The Set
% Method worked well only in case of zoom. Never got it worked correctly on
% pan method. Finally realized that the demo is awesome anyway and gave up on
% this.
cla(S.VisIm);
S.VisIm = axes();
xlim([S.XMin,S.XMax]);            % Set Limits
ylim([S.YMin,S.YMax]);            % Set Limits
S.VisIm.Position = [S.ButtonGroupWidth + 2* S.BGSpacing,S.BGSpacing,1 - 3*S.BGSpacing - S.ButtonGroupWidth,1-2*S.BGSpacing];  
S.VisIm.Color = 0.7*ones(1,3);    % Axes Background color 
S.VisIm.Clipping = 'off';         % Annnoying
S.VisIm.BoxStyle = 'full';        % Full 
S.VisImSC = imagesc(S.VisIm,[S.VisIm.XLim],[S.VisIm.YLim],Image);
S.VisIm.XLabel = [];              % Disable X Labels
S.VisIm.YLabel = [];              % Disable Y Labels   
S.VisIm.XTick = [];               % Disable X-Ticks
S.VisIm.YTick = [];               % Disable Y-Ticks





end