#pragma once

// --------------------------------
// ------ CUDA Include Files ------
#include "cuda_runtime.h"
#include "device_launch_parameters.h"
#include <cufft.h>

enum ComputingType {
	JULIA = 0,
	MANDEL = 1
};

enum ComplexOutput {
	REAL = 0,   // Takes Real from complex input
	IMAG = 1,   // Takes Imaginary from complex input
	ABS = 2,    // Takes Absolute from Complex input
	ANGLE = 3,  // Takes Angle of the Complex Number
};

// -------------------------------------
// ------ Kernel Call Definitions ------
__global__ void JuliaKernel				(cufftComplex* Plane, cufftComplex C, int* Iterations, float Threshold, int NumIter, int Nx, int Ny);
__global__ void IntToFloat				(int* Data_In, cufftReal* Data_Out, int Nx, int Ny);
__global__ void IntToComplex			(int* Data_In, cufftComplex* Data_Out, int Nx, int Ny);
__global__ void ComplexToReal			(cufftComplex* Data_In, cufftReal* Data_Out, int Nx, int Ny);
__global__ void ComplexMultiply			(cufftComplex* In, cufftComplex* Out, int Nx, int Ny);
__global__ void DeployKernel			(cufftReal* Kernel, cufftComplex* Data, int Nx, int Ny, int Hx, int Hy);
__global__ void CropImage				(cufftComplex* BigImage, cufftComplex* SmallImage, int Nx, int Ny, int Hx, int Hy);
__global__ void CropImage				(cufftReal* BigImage, cufftReal* SmallImage, int Nx, int Ny, int Hx, int Hy);
__global__ void MandelBrotKernel		(cufftComplex* C, cufftReal* X, cufftReal* Y, int* Iterations, float Threshold, int NumIter, int Nx, int Ny);
__global__ void JuliaMandelInitialize	(cufftComplex* Plane, float xStart, float xStop, float yStart, float yStop, int Nx, int Ny);
__global__ void Col2Row					(cufftReal* in, cufftReal* out, int Nx, int Ny);
__global__ void Row2Col					(cufftReal* Data_In, cufftReal* Data_Out, int Nx, int Ny);
__global__ void ComplexToCustom			(cufftComplex* DIn, cufftReal* DOut, ComplexOutput Out, int Nx, int Ny);
__global__ void ShiftRows2D				(cufftComplex* in, cufftComplex* out, int Nx, int Ny, int XHold, int YHold, int dir);
__global__ void ShiftCols2D				(cufftComplex* in, cufftComplex* out, int Nx, int Ny, int XHold, int YHold, int dir);
__global__ void ComplexToCustom			(cufftReal* DIn_R, cufftReal* DIn_I, cufftReal* DOut, ComplexOutput Out, int Nx, int Ny);